%%% This program generates draws for historical decompositions
%%% By Christiane Baumeister and James D. Hamilton (Aug 2015)

% To be run after running main_general.m and irf.m, which should have set variables including:
%  psi_invA = [n x n x (hmax+nlags) x nuse] array which includes structural
%     impulse-response matrices
%     dimension 1: variable affected
%     dimension 2: shock whose effect is sought
%     dimension 3: hmax + s component is effect of shock after s periods (s = 0,1,..,hmax-1)
%     dimension 4: indexes the Monte Carlo draw of the parameters
%  note psi_invA is the cumulation of IRFs if kcum = 1
%  YY = (T x n) matrix of observations on dependent variable
%  XX = (T x k) matrix of observations on explanatory variables
%  a_post = [na x (ndraws - nburn)] matrix of draws for elements that go into (n x n) matrix A
%  Bdraw = (n x k x nuse) array of draws for the (n x k) matrix B

% output:
% QH = (n x n x T x nuse) array of components of y attributable to each shock
%     dimension 1: variable affected
%     dimension 2: shock whose effect is sought
%     dimension 3: date of observation
%     dimension 4: indexes the Monte Carlo draw of the parameters

QH = zeros(n,n,T,nuse);

% ===========================================================
% 'Generating draws for historical decomposition'
isim = 0;
while isim < nuse
    isim = isim + 1;
    if (isim/10000) == floor(isim/10000)
        isim
    end

    % ============================================================
    % construct U = (T x n) matrix containing history of structural residuals for this draw
    A = setA(a_post(:,isim),n);  
    B = squeeze(Bdraw(:,:,isim));
    U = YY*A' - XX*B';

    for j = 1:n
        % =============================================
        % find historical contribution of structural shock j
        ulast = U(:,j);
        uj = zeros(T,n);
        for hsim = 0:hmax
            uj = uj + ulast*squeeze(psi_invA(:,j,nlags+hsim,isim))';
            ulast = [0; ulast(1:end-1)];
        end
    QH(:,j,:,isim) = uj';
    end
end

% ============================================================
% calculate median and percentiles
index0 = round(0.5*nuse);
index1 = round(0.025*nuse);
index2 = round((1 - 0.025)*nuse);
index3 = round(0.16*nuse);
index4 = round((1 - 0.16)*nuse);

QH0 = zeros(n,n,T);
QH1 = QH0;
QH2 = QH0;
QH3 = QH0;
QH4 = QH0;

for i = 1:n
    for j = 1:n
        for t = 1:T
           K = squeeze(QH(i,j,t,:));
           K = sort(K);
           %QH0(i,j,t) = mean(K);     %posterior mean 
           QH0(i,j,t) = K(index0);    %posterior median
           QH1(i,j,t) = K(index1);
           QH2(i,j,t) = K(index2);
           QH3(i,j,t) = K(index3);
           QH4(i,j,t) = K(index4);
        end
    end
end
    
% =========================================================
% Report results
display('Columns report contributions of different shocks for indicated variable')   
varnames2 = char(' output gap',' inflation',' fed funds rate');
for i = 1:n
    display(strcat('median for ',varnames2(i,:)))
    format bank
    [time squeeze(QH0(i,:,:))']
    format short
end
    
% ==================================================
% display historical decompositions as graphs
ct=1;
figure(5)
for i=1:n
    
    yy = YY(:,i) - mean(YY(:,i));
          
    for j = 1:n
       
       subplot(3,3,ct)
       temp1=[squeeze(QH0(i,j,:)) squeeze(QH3(i,j,:)) squeeze(QH4(i,j,:))];
       temp2=[squeeze(QH1(i,j,:)) squeeze(QH2(i,j,:))];
       plotx2(temp1,time);
       box on;
       plot(time,yy,'r--','LineWidth',2), hold on, 
       plot(time,temp1(:,1),'b','LineWidth',2), hold on, 
       plot(time,temp2,'b:','LineWidth',2), hold on, 
       plot(time,zeros(size(time,1),1),'k:','LineWidth',1) 
       axis([date_start date_end -8 8])
       temptitle = strcat('Effect of ',shocknames(j),' shocks on ',varnames(i));
       title(temptitle)
       ct=ct+1;
    end
    
end
     