%%% Code to replicate results of "Inference in Structural Vector Autoregressions 
%%% When the Identifying Assumptions are Not Fully Believed: Re-evaluating 
%%% the Role of Monetary Policy in Economic Fluctuations"
%%% for larger dimensional system (6-variable VAR)

%%% By Christiane Baumeister and James D. Hamilton (April 2018)

clear;
clc;
   
ndraws=2e6;     %number of MH iterations 
nburn=1e6;      %number of burn-in draws

% NOTE:
% The parameter chsi is the tuning parameter for the MH step and has to be
% re-set for each application such that it implies a 30% acceptance ratio.
% If acceptance ratio is too high, make chsi bigger; if acceptance ratio is
% too low, make chsi smaller.
nA = 30;    % number of elements in A for which priors are formed
chsi = 0.007*eye(nA);   

prior=0;   
           
% NOTE:
% You can change priors for elements in A and their informativeness in: setPrior2.m
% You can change priors on impact signs/magnitudes in: logP.m

%================================================
% This section reads in data in the form of
%      yall = full data set from which lags will be constructed
%      tstart = element of yall that will be obs 1 in regression
%      tend = element of yall that will be obs T in regression
%      T = number of usable observations for variable on LHS of regression
%        = tend - tstart + 1
%      YY = (T x n) matrix of T observations on each of n variables
%         = elements of yall for rows tstart through tend
VARmodel = 'large';
readData2;
YY

% ==================================================
% construct matrix of lags 
%      XX = (T x k) matrix of observations on k different regressors
%    (this section should work as is for all applications)

T = size(YY,1);     % T is sample size
n = size(YY,2);     % n is number of equations

if nlags > 0
    XX = yall(tstart-1:tend-1,:);
    ilags = 1;
    while ilags < nlags
        ilags = ilags + 1;
        XX = [XX yall(tstart-ilags:tend-ilags,:)];
    end
    XX = [XX ones(T,1)];
else
    XX = ones(T,1);     % always include constant in XX 
end

k = size(XX,2);         % k is number of regressors

omegahat = (YY'*YY - YY'*XX*inv(XX'*XX)*XX'*YY)/T;
omegahat 

%=====================================================================
% set values that characterize the priors and how they will be plotted
setPrior2;       
xbound = 6;
ybound = 2;
steps = 0.01;

%===============================================================
% calculate variance-covariance matrix of univariate AR residuals
e = zeros(T,n);
i = 0;
while i < n
   i = i+1;
   if nlags > 0
      ylags = yall(tstart-1:tend-1,i);
      ilags = 1;
      while ilags < nlags
         ilags = ilags + 1;
         ylags = [ylags yall(tstart-ilags:tend-ilags,i)];
      end
      ylags = [ylags ones(T,1)];
   else
      ylags = ones(T,1);
   end
   e(:,i) = yall(tstart:tend,i) - ylags*inv(ylags'*ylags)*ylags'*yall(tstart:tend,i);
end
Sstar = e'*e/T;
Sstar

%============================================================
% Calculate inverse of Mtilde for standard Minn prior
v1 = 1:nlags;
v1 = v1'.^(-2*lambda1);
v2 = 1./diag(Sstar);
v3 = kron(v1,v2);
v3 = lambda0^2*[v3; lambda3^2];
v3 = 1./v3;
Mtildeinv = diag(v3);

%============================================================
% calculate parameters that characterize posteriors
%    (this section should not need to be changed for different
%    applications)

kappastar = kappa + (T/2); 
if prior==0
    ytilde = YY'*YY + eta*Mtildeinv*eta';
    yxtilde = YY'*XX + eta*Mtildeinv;
else
    ytilde = eta*Mtildeinv*eta';
    yxtilde = eta*Mtildeinv;
end
    
xtildei = zeros(k,k,n);
i = 0;
while i < n
    i = i+1;
    if longA(i) == 0
        if prior == 0
            xtildei(:,:,i) = inv(XX'*XX + Mtildeinv);
        else 
            xtildei(:,:,i) = inv(Mtildeinv);
        end
        
    else
        if prior == 0
            xtildei(:,:,i) = inv(XX'*XX + Mtildeinv + Ri'*Ri/Vi);
        else
            xtildei(:,:,i) = inv(Mtildeinv + Ri'*Ri/Vi);
        end
    end
end


seednumber=140778;
rand('seed',seednumber);
randn('seed',seednumber);

%=====================================================
% calculate prior densities to plot against posteriors

z1=-xbound:steps:xbound;         %grid for parameters to define points where density is evaluated
pdf_prior = zeros(nA-1,size(z1,2));

% t-distributed priors
i = 0;
while i < nA-1
     i = i + 1;
     z2 = (z1 - cA(i))/sigA(i);
     if signA(i) == 1
 	pdf_prior(i,:) = ((z1 > 0.0) .* (tpdf(z2,nuA(i))/sigA(i))) / (1 - tcdf(-cA(i)/sigA(i),nuA(i)));
     elseif signA(i) == -1
	pdf_prior(i,:) = ((z1 < 0.0) .* (tpdf(z2,nuA(i))/sigA(i))) / tcdf(-cA(i)/sigA(i),nuA(i));
     elseif signA(i) == 0
	pdf_prior(i,:) = tpdf(z2,nuA(i)) / sigA(i);
     end 
end

% Beta-distributed prior
f1=0:0.001:1;
pdf_prior_rho=beta_prior(f1,A_rho,B_rho);  % interest rate smoothing

%=======================================================
% find posterior mode and hessian for scaling if desired
k_opt_scale = 1;   
	% k_opt_scale = 1 for optimal scaling
	% k_opt_scale = 0 for fixed scaling

if k_opt_scale == 1
   % create anonymous function whose only argument is theta
   A_params = [cA sigA nuA signA];
   f_anon = @(theta)post_val(theta,A_params,A_rho,B_rho,longA,kappa,T,omegahat,...
       Sstar,ytilde,xtildei,yxtilde,Ri,Vi,prior);

   % find posterior mode
   theta_zero = [cA;(A_rho-1)/(A_rho+B_rho-2)];
   [theta_max,val_max,exitm,om,gm,hm] = fminunc(f_anon,theta_zero)
   'posterior mode is'
   theta_max

   % find hessian of log posterior
   if min(eig(inv(hm))) > 0
        opt_scale = chol(inv(hm))';
   else
        opt_scale = eye(nA);
   end
   
   if min(sign(theta_max(1:end-1,1)).*signA) >=0 && theta_max(end,1)>=0 && theta_max(end,1)<=1
       theta_start = theta_max;
   else
       theta_start = theta_zero;
   end

elseif k_opt_scale == 0
   opt_scale = eye(nA);
   theta_start = theta_zero;
end


%==================================================
% RW-MH algorithm to generate A

naccept=0;
count=0;

% set initial value
   Q1 = zeros(n,1);
   a_old = theta_start;
   A = setA(a_old,n);
   Q = A*omegahat*A';
   tau = kappa.*diag(A*Sstar*A');
   zetastar = zeros(n,1);
   i = 0;
   while i < n
       i = i+1;
       ytildei = A(i,:)*ytilde*A(i,:)';
       yxtildei = A(i,:)*yxtilde;
       if longA(i) == 1
           ri = a_old(end,1);
           ytildei = ytildei + ri^2/Vi;
           yxtildei = yxtildei + ri*Ri/Vi;
       end
       zetastar(i) = ytildei - yxtildei*xtildei(:,:,i)*yxtildei';
   end
   
   if prior == 0
       ptarget_old = logP(a_old,cA,sigA,nuA,signA,A_rho,B_rho,n) + (T/2)*log(det(Q)) ...
                   - kappastar'*log((2*tau/T) + zetastar/T) + kappa'*log(tau);
   else 
       ptarget_old = logP(a_old,cA,sigA,nuA,signA,A_rho,B_rho,n);
   end
   
   
% Store posterior distribution (after burn-in)
a_post=zeros(nA,ndraws-nburn);
zeta_post=zeros(n,ndraws-nburn);
mstar_post=zeros(n,k,ndraws-nburn);

while count<ndraws 
    count=count+1;
    if (count/10000) == floor(count/10000)
        count
    end
    
    a_new=a_old+chsi*opt_scale*randn(nA,1)/sqrt(0.5*(randn(1)^2 + randn(1)^2));
    A = setA(a_new,n);
    H = inv(A);
    
 if min(sign(a_new(1:end-1,1)).*signA) >=0 && a_new(end,1)>=0 && a_new(end,1)<=1 && ...
         H(1,3)<0 && H(2,3)<0 && H(3,3)>0 && H(4,3)>0 && H(5,3)<0 && H(6,3)<0
	
    Q = A*omegahat*A';
    tau = kappa.*diag(A*Sstar*A');
    zeta_new = zeros(n,1);
    mstar_new = zeros(n,k);
    i = 0;
    while i < n
       i = i+1;
       ytildei = A(i,:)*ytilde*A(i,:)';
       yxtildei = A(i,:)*yxtilde;
       if longA(i) == 1
           ri = a_new(end,1);
           ytildei = ytildei + ri^2/Vi;
           yxtildei = yxtildei + ri*Ri/Vi;
       end
       zeta_new(i) = ytildei - yxtildei*xtildei(:,:,i)*yxtildei';
       mstar_new(i,:) = yxtildei*xtildei(:,:,i);
    end
    
    if prior == 0
        ptarget_new = logP(a_new,cA,sigA,nuA,signA,A_rho,B_rho,n) + (T/2)*log(det(Q)) ...
                    - kappastar'*log((2*tau/T) + zeta_new/T) + kappa'*log(tau);
    else 
        ptarget_new = logP(a_new,cA,sigA,nuA,signA,A_rho,B_rho,n);
    end
    
    accept=min([exp(ptarget_new - ptarget_old);1]);
       u=rand(1);                      %draw from a uniform distribution
       if u<=accept
          a_old=a_new;                %we retain the new draw
          zeta_old = zeta_new;
          mstar_old = mstar_new;
	      ptarget_old = ptarget_new;
          naccept=naccept+1;          %count the number of acceptances
       end
       
 end
   
 %Store results after burn-in    
    if count>nburn
        a_post(:,count-nburn)=a_old;
        zeta_post(:,count-nburn) = zeta_old;
        mstar_post(:,:,count-nburn) = mstar_old;
    end 
    
end

% Compute acceptance ratio of RW-MH algorithm
  acceptance_ratio=naccept/ndraws;
  disp(['Acceptance ratio:' num2str(acceptance_ratio)])

% ====================================================
% analyze convergence of the chain
% p1=0.1;   %first 10% of the sample (for Geweke's (1992) convergence diagnostic)
% p2=0.5;   %last 50% of the sample
% autoc = convergence_diagnostics(a_post,p1,p2);

% produce figures
hmax = 21;    % hmax - 1 is the maximum horizon sought
irf;          % computes posterior median of impulse responses and 68% and 95% credibility sets
varDecomp;    % computes posterior median of variance decomposition and 95% credibility sets
    


